@@ -38,8 +38,8 @@ function [idx,score] = fss(D,cl,N,MODE)
 %   Volume 86, Issue 1, 15 March 2007, Pages 68-81 
 %   http://dx.doi.org/10.1016/j.chemolab.2006.08.007
 
-%	$Id: fss.m 8223 2011-04-20 09:16:06Z schloegl $
-%	Copyright (C) 2009,2010 by Alois Schloegl <alois.schloegl@gmail.com>	
+%	$Id: fss.m 9104 2011-11-15 15:14:10Z carandraug $
+%	Copyright (C) 2009,2010 by Alois Schloegl <alois.schloegl@gmail.com>
 %       This function is part of the NaN-toolbox
 %       http://pub.ist.ac.at/~schloegl/matlab/NaN/
 
@@ -56,91 +56,89 @@ function [idx,score] = fss(D,cl,N,MODE)
 %    You should have received a copy of the GNU General Public License
 %    along with this program; If not, see <http://www.gnu.org/licenses/>.
 
-
-if nargin<3, 
-	MODE = []; 
-	N = [];
-elseif ischar(N)
-	MODE = N; 
-	N = [];	
-elseif nargin<4,
-	MODE = [];	
-end; 
-
-if isempty(N), N = size(D,2); end;
-score = repmat(NaN,1,size(D,2));
-
-if 0, %strcmpi(MODE,'MRMR') || strcmpi(MODE,'MID') || strcmpi(MODE,'MIQ');
-        %% RMRM/MID/MIQ is not supported
-        %% TODO: FIXME 
-         
-        [tmp,t] = sort([cl,D]);
-        cl = t(:,1:size(cl,2)); 
-        D  = t(:,1:size(D,2)); 
-        for k = 1:N,
-                V(k) = mi(cl, D(:,k));
-
-                for m = 1:N, 
-                        W(k,m) = mi(D(:,m), D(:,k));
-                end;
-                MID(k) = V(k) - mean(W(k,:));
-                MIQ(k) = V(k) / mean(W(k,:));
+        if nargin<3
+                MODE = [];
+                N = [];
+        elseif ischar(N)
+                MODE = N;
+                N = [];
+        elseif nargin<4,
+                MODE = [];
         end
 
-        if  strcmpi(MODE,'MIQ')
-                [score,idx] = sort(MIQ,[],'descend'); 
-        else        
-                [score,idx] = sort(MID,[],'descend'); 
-        end         
+        if isempty(N), N = size(D,2); end
+        score = repmat(NaN,1,size(D,2));
 
-elseif strcmpi(MODE,'FSDD'); 
-        [b,i,j]=unique(cl); 
-        for k=1:length(b)
-                n(k,1) = sum(j==k);
-                m(k,:) = mean(D(j==k,:),1);
-                v(k,:) = var(D(j==k,:),1);
-        end; 
-        m0 = mean(m,1,n); 
-        v0 = var(D,[],1);
-        s2 = mean(m.^2,1,n) - m0.^2;        
-        score = (s2 - 2*mean(v,1,n)) ./ v0;
-        [t,idx] = sort(-score); 
- 
-elseif isempty(MODE) || strcmpi(MODE,'rank') || strcmpi(MODE,'Pearson')
-        cl = cat2bin(cl); 
-        if strcmpi(MODE,'rank'),
-                [tmp,D] = sort(D,1);
-        end; 
-        idx = repmat(NaN,1,N);
-        for k = 1:N,
-	        f = isnan(score);
+        if 0, %strcmpi(MODE,'MRMR') || strcmpi(MODE,'MID') || strcmpi(MODE,'MIQ');
+                %% RMRM/MID/MIQ is not supported
+                %% TODO: FIXME 
+                 
+                [tmp,t] = sort([cl,D]);
+                cl = t(:,1:size(cl,2)); 
+                D  = t(:,1:size(D,2)); 
+                for k = 1:N,
+                        V(k) = mi(cl, D(:,k));
 
-                %%%%% compute partial correlation (X,Y|Z)
-       	        % r = partcorrcoef(cl, D(:,f), D(:,~f)); % obsolete, not very robust
+                        for m = 1:N,
+                                W(k,m) = mi(D(:,m), D(:,k));
+                        end
+                        MID(k) = V(k) - mean(W(k,:));
+                        MIQ(k) = V(k) / mean(W(k,:));
+                end
 
-       	        %% this is a more robust version 
-                X = cl; Y = D(:,f); Z = D(:,~f);
-                if (k>1)
-       	                X = X-Z*(Z\X);
-                        Y = Y-Z*(Z\Y);
-                end;
-                r = corrcoef(X,Y);
+                if  strcmpi(MODE,'MIQ')
+                        [score,idx] = sort(MIQ,[],'descend');
+                else
+                        [score,idx] = sort(MID,[],'descend');
+                end
 
-	        [s,ix] = max(sumsq(r,1));
-        	f      = find(f);
-        	idx(k) = f(ix);
-        	score(idx(k)) = s;
-        end;
+        elseif strcmpi(MODE,'FSDD');
+                [b,i,j]=unique(cl);
+                for k=1:length(b)
+                        n(k,1) = sum(j==k);
+                        m(k,:) = mean(D(j==k,:),1);
+                        v(k,:) = var(D(j==k,:),1);
+                end
+                m0 = mean(m,1,n);
+                v0 = var(D,[],1);
+                s2 = mean(m.^2,1,n) - m0.^2;
+                score = (s2 - 2*mean(v,1,n)) ./ v0;
+                [t,idx] = sort(-score);
 
+        elseif isempty(MODE) || strcmpi(MODE,'rank') || strcmpi(MODE,'Pearson')
+                cl = cat2bin(cl);
+                if strcmpi(MODE,'rank'),
+                        [tmp,D] = sort(D,1);
+                end
+                idx = repmat(NaN,1,N);
+                for k = 1:N,
+                        f = isnan(score);
+
+                        %%%%% compute partial correlation (X,Y|Z)
+                        % r = partcorrcoef(cl, D(:,f), D(:,~f)); % obsolete, not very robust
+
+                        %% this is a more robust version 
+                        X = cl; Y = D(:,f); Z = D(:,~f);
+                        if (k>1)
+                                X = X-Z*(Z\X);
+                                Y = Y-Z*(Z\Y);
+                        end
+                        r = corrcoef(X,Y);
+
+                        [s,ix] = max(sumsq(r,1));
+                        f      = find(f);
+                        idx(k) = f(ix);
+                        score(idx(k)) = s;
+                end
+
+        end
 end
 
 function I = mi(x,y)
         ix  = ~any(isnan([x,y]),2);
-        H   = sparse(x(ix),y(ix)); 
+        H   = sparse(x(ix),y(ix));
         pij = H./sum(ix); 
         Iij = pij.*log2(pij./(sum(pij,2)*sum(pij,1)));
-        Iij(isnan(Iij)) = 0; 
+        Iij(isnan(Iij)) = 0;
         I = sum(Iij(:));
 end
-
-end
