@@ -24,7 +24,7 @@
 %    You should have received a copy of the GNU General Public License
 %    along with this program; If not, see <http://www.gnu.org/licenses/>.
 
-%	$Id: nantest.m 8223 2011-04-20 09:16:06Z schloegl $
+%	$Id: nantest.m 12844 2015-07-02 21:04:17Z schloegl $
 %	Copyright (C) 2000-2004,2009 by Alois Schloegl <alois.schloegl@gmail.com>
 %       This script is part of the NaN-toolbox
 %       http://pub.ist.ac.at/~schloegl/matlab/NaN/
@@ -62,7 +62,10 @@ if exist('normcdf','file')==2,
         end;
 end;
 
-if exist('norminv','file')==2,
+if ~(any(exist('erfinv') == [2,5]))
+       fprintf(1,'ERFINV is not available\n');
+
+elseif exist('norminv','file')==2,
         p = [-inf,-.2,0,.2,.5,1,2,inf,nan];
         q(3) = sum(~isnan(norminv(p,2,0)))<4;
         if q(3),
@@ -200,9 +203,9 @@ if all([])==0,
 	fprintf(1,'WARNING: ALL([]) returns 0 instead of 1\n');
 end;
 	
-%%%%% SORT - this was once a problem in Octave Version < 2.1.36 %%%%
+%%%%% SORT - this was once a problem in Octave Version < 2.1.36, and still is in FreeMat 4.0 %%%%
 if ~all(isnan(sort([3,4,NaN,3,4,NaN]))==[0,0,0,0,1,1]), 
-        warning('Warning: SORT does not handle NaN.');
+        warning('Warning: SORT does not properly handle NaN.');
 end;
 
 %%%%% commutativity of 0*NaN	%%% This test adresses a problem in Octave
@@ -224,8 +227,23 @@ end;
 %(roots([2e-37,-2,2])-[1e37;1])
 %%%%% check nan/nan   %% this test addresses a problem in Matlab 5.3, 6.1 & 6.5
 p    = 4;
-tmp1 = repmat(nan,p)/repmat(nan,p);
-tmp2 = repmat(nan,p)\repmat(nan,p);
+tmp1 = repmat(nan, 4);
+tmp2 = repmat(nan, 4);
+if ispc
+        % Octave 4.0.0 on Windows crashes, therefore the test is disabled
+	warning('mrdivide (repmat(nan,4), repmat(nan,4)) and mldivide (repmat(nan,4), repmat(nan,4)) not tested because it might crash Octave on Windows.\n');	
+else        
+try
+	tmp1 = repmat(nan,p) / repmat(nan,p);
+catch   % exception error in Octave 3.8.2 and later of debian wheezy
+	fprintf(2,'mrdivide (repmat(nan,4), repmat(nan,4)) fails with an exception\n');	
+end;
+try
+	tmp2 = repmat(nan,p) \ repmat(nan,p);
+catch   % exception error in Octave 3.8.2 and later of debian wheezy
+	fprintf(2,'mldivide (repmat(nan,4), repmat(nan,4)) fails with an exception\n');
+end
+end;
 tmp3 = repmat(0,p)/repmat(0,p);
 tmp4 = repmat(0,p)\repmat(0,p);
 tmp5 = repmat(0,p)*repmat(inf,p);
@@ -267,16 +285,16 @@ tmp  = [tmp1;tmp2;tmp3;tmp4;tmp5;tmp6;tmp7;tmp8];
 
 
 %%%%% QUANTILE TEST 
-d = [1 1 2 2 4 4 10 700]; 
+d = [1 1 2 2 4 4 10 700]'; 
 q = [-1,0,.05,.1,.25,.49,.5,.51,.75,.8, .999999,1,2];
 r = [ NaN, 1, 1, 1, 1.5, 2, 3, 4, 7, 10, 700, 700, NaN]; 
-if any( quantile(d, q) -  r>0)
+if any( quantile(d, q)' -  r>0)
 	fprintf(1,'Quantile(1): failed\n');
 else
 	fprintf(1,'Quantile(1): OK\n'); 
 end; 
 if exist('histo3','file')
-	H = histo3(d');
+	H = histo3(d);
 else
 	H.X = [1;2;4;10;700];
 	H.H = [2;2;2;1;1];
