@@ -84,7 +84,7 @@ function [R,CC]=xval(D,classlabel,MODE,arg4)
 % 
 % You should have received a copy of the GNU General Public License
 % along with this program; if not, write to the Free Software
-% Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+% Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
 
 if (nargin<3) || isempty(MODE),
 	MODE = 'LDA';
@@ -162,7 +162,7 @@ for k = 1:max(NG),
 		CC = train_sc(D(ix,:), C(ix), MODE, W(ix));
 	end;
  	ix = ix0(NG(ix0)==k);
-	r  = test_sc(CC, D(ix,:))
+	r  = test_sc(CC, D(ix,:));
 	cl(ix,1) = r.classlabel;
 end; 
 
