@@ -1,7 +1,5 @@
 /*
 //-------------------------------------------------------------------
-#pragma hdrstop
-//-------------------------------------------------------------------
 //   C-MEX implementation of COVM - this function is part of the NaN-toolbox. 
 //
 //
@@ -34,16 +32,16 @@
 // - NN = real(~isnan(X)')*sparse(diag(W))*real(~isnan(Y))   count of valid (non-NaN) elements
 //        computed more efficiently 
 //
-//    $Id: covm_mex.cpp 6771 2010-01-14 17:50:16Z schloegl $
-//    Copyright (C) 2009 Alois Schloegl <a.schloegl@ieee.org>
+//    $Id: covm_mex.cpp 12826 2015-06-18 15:09:49Z schloegl $
+//    Copyright (C) 2009,2010,2011 Alois Schloegl <alois.schloegl@gmail.com>
 //    This function is part of the NaN-toolbox
-//    http://hci.tugraz.at/~schloegl/matlab/NaN/
+//    http://pub.ist.ac.at/~schloegl/matlab/NaN/
 //
 //-------------------------------------------------------------------
 */
 
 #ifdef __GNUC__ 
-        #include <inttypes.h>
+        #include <stdint.h>
 #endif
 #include <math.h>
 #include "mex.h"
@@ -53,13 +51,13 @@
 
 void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const mxArray *PInputs[]) 
 {
-    	double 		*X0,*Y0=NULL,*X,*Y,*W=NULL;
-    	double	 	*CC,cc,nw;
-    	double 		*NN=NULL;
+    	double 		*X0=NULL, *Y0=NULL, *W=NULL;
+    	double	 	*CC;
+    	double 		*NN = NULL;
 
-    	size_t		rX,cX,rY,cY,nW = 0;
-    	size_t    	i,j,k,nn; 
-	char	 	flag_isNaN = 0, flag_speed=0;
+    	size_t		rX,cX,rY,cY;
+    	size_t    	i; 
+	char	 	flag_isNaN = 0;
         int             ACC_LEVEL;
 
 	/*********** check input arguments *****************/
@@ -85,10 +83,10 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 
 
 	// get 1st argument
-	if(mxIsDouble(PInputs[0]) && !mxIsComplex(PInputs[0]))
+	if(mxIsDouble(PInputs[0]) && !mxIsComplex(PInputs[0]) && !mxIsSparse(PInputs[0]) )
 		X0  = mxGetPr(PInputs[0]);
 	else 	
-		mexErrMsgTxt("First argument must be REAL/DOUBLE.");
+		mexErrMsgTxt("First argument must be non-sparse REAL/DOUBLE.");
 	rX = mxGetM(PInputs[0]);		
 	cX = mxGetN(PInputs[0]);		
 		
@@ -108,7 +106,7 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
     	// get weight vector for weighted sumskipnan 
        	if  (PInputCount > 3)	{
 		// get 4th argument
-		nW = mxGetNumberOfElements(PInputs[3]);		
+		size_t nW = mxGetNumberOfElements(PInputs[3]);		
 		if (!nW) 
 			; 
 		else if (nW == rX) 	
@@ -209,6 +207,9 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 	}
 	
 #else
+
+   #pragma omp parallel 
+   {
 #ifdef __GNUC__
    if (ACC_LEVEL == 0) 
 #endif
@@ -219,12 +220,14 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 	if ( (X0 != Y0) || (cX != cY) )
 		/******** X!=Y, output is not symetric *******/	
 	    if (W) /* weighted version */
-	    for (i=0; i<cX; i++)
-	    for (j=0; j<cY; j++) {
-		X = X0+i*rX;
-		Y = Y0+j*rY;
-		cc=0.0;
-		nw=0.0;
+            #pragma omp for schedule(dynamic) nowait
+	    for (i = 0; i < cX * cY; i++)
+	    {
+		double *X = X0 + (i%cX) * rX;
+		double *Y = Y0 + (i/cX) * rY;
+		double cc = 0.0;
+		double nw = 0.0;
+		size_t k; 
 		for (k=0; k<rX; k++) {
 			double z = X[k]*Y[k];
 			if (isnan(z)) {
@@ -236,17 +239,19 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 			cc += z*W[k];
 			nw += W[k];
 		}	
-		CC[i+j*cX] = cc; 
+		CC[i] = cc; 
 		if (NN != NULL) 
-			NN[i+j*cX] = nw; 
+			NN[i] = nw; 
 	    }
 	    else /* no weights, all weights are 1 */
-  	    for (i=0; i<cX; i++)
-	    for (j=0; j<cY; j++) {
-		X = X0+i*rX;
-		Y = Y0+j*rY;
-		cc=0.0;
-		nn=0;
+            #pragma omp for schedule(dynamic) nowait
+	    for (i = 0; i < cX * cY; i++)
+	    {
+		double *X = X0 + (i%cX) * rX;
+		double *Y = Y0 + (i/cX) * rY;
+		double cc = 0.0;
+		size_t nn = 0;
+		size_t k; 
 		for (k=0; k<rX; k++) {
 			double z = X[k]*Y[k];
 			if (isnan(z)) {
@@ -258,19 +263,24 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 			cc += z;
 			nn++;
 		}	
-		CC[i+j*cX] = cc; 
+		CC[i] = cc; 
 		if (NN != NULL) 
-			NN[i+j*cX] = (double)nn; 
+			NN[i] = (double)nn; 
 	    }
 	else // if (X0==Y0) && (cX==cY)
 		/******** X==Y, output is symetric *******/	
 	    if (W) /* weighted version */
-	    for (i=0; i<cX; i++)
-	    for (j=i; j<cY; j++) {
-		X = X0+i*rX;
-		Y = Y0+j*rY;
-		cc=0.0;
-		nw=0.0;
+            #pragma omp for schedule(dynamic) nowait
+	    for (i = 0; i < cX * cY; i++)
+	    {
+		size_t ii = i%cX;
+		size_t jj = i/cX;
+	        if (ii < jj) continue;     
+		double *X = X0 + ii * rX;
+		double *Y = Y0 + jj * rY;
+		double cc = 0.0;
+		double nw = 0.0;
+		size_t k; 
 		for (k=0; k<rX; k++) {
 			double z = X[k]*Y[k];
 			if (isnan(z)) {
@@ -281,21 +291,27 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 			}
 			cc += z*W[k];
 			nw += W[k];
-		}	
-		CC[i+j*cX] = cc; 
-		CC[j+i*cX] = cc; 
+		}
+		size_t j = jj + ii*cX;
+		CC[i] = cc; 
+		CC[j] = cc; 
 		if (NN != NULL) {
-			NN[i+j*cX] = nw; 
-			NN[j+i*cX] = nw; 
-		}	
+			NN[i] = nw; 
+			NN[j] = nw; 
+		}
 	    }
 	    else /* no weights, all weights are 1 */
-	    for (i=0; i<cX; i++)
-	    for (j=i; j<cY; j++) {
-		X = X0+i*rX;
-		Y = Y0+j*rY;
-		cc=0.0;
-		nn=0;
+            #pragma omp for schedule(dynamic) nowait
+	    for (i = 0; i < cX * cY; i++)
+	    {
+		size_t ii = i%cX;
+		size_t jj = i/cX;
+	        if (ii < jj) continue;     
+		double *X = X0 + ii * rX;
+		double *Y = Y0 + jj * rY;
+		double cc = 0.0;
+		size_t nn = 0;
+		size_t k; 
 		for (k=0; k<rX; k++) {
 			double z = X[k]*Y[k];
 			if (isnan(z)) {
@@ -307,11 +323,12 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 			cc += z;
 			nn++;
 		}	
-		CC[i+j*cX] = cc; 
-		CC[j+i*cX] = cc; 
+		size_t j = jj + ii*cX;
+		CC[i] = cc; 
+		CC[j] = cc; 
 		if (NN != NULL) {
-			NN[i+j*cX] = (double)nn; 
-			NN[j+i*cX] = (double)nn; 
+			NN[i] = (double)nn; 
+			NN[j] = (double)nn; 
 		}	
 	    }
 
@@ -326,12 +343,14 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 	if ( (X0 != Y0) || (cX != cY) )
 		/******** X!=Y, output is not symetric *******/	
 	    if (W) /* weighted version */
-	    for (i=0; i<cX; i++)
-	    for (j=0; j<cY; j++) {
-		X = X0+i*rX;
-		Y = Y0+j*rY;
+            #pragma omp for schedule(dynamic) nowait
+	    for (i = 0; i < cX * cY; i++)
+	    {
+		double *X = X0 + (i%cX) * rX;
+		double *Y = Y0 + (i/cX) * rY;
 		long double cc=0.0;
 		long double nn=0.0;
+		size_t k; 
 		for (k=0; k<rX; k++) {
 			long double z = ((long double)X[k])*Y[k];
 			if (isnan(z)) {
@@ -343,17 +362,19 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 			cc += z*W[k];
 			nn += W[k];
 		}	
-		CC[i+j*cX] = cc; 
+		CC[i] = cc; 
 		if (NN != NULL) 
-			NN[i+j*cX] = nn; 
+			NN[i] = nn; 
 	    }
 	    else /* no weights, all weights are 1 */
-  	    for (i=0; i<cX; i++)
-	    for (j=0; j<cY; j++) {
-		X = X0+i*rX;
-		Y = Y0+j*rY;
+            #pragma omp for schedule(dynamic) nowait
+	    for (i = 0; i < cX * cY; i++)
+	    {
+		double *X = X0 + (i%cX) * rX;
+		double *Y = Y0 + (i/cX) * rY;
 		long double cc=0.0;
 		size_t nn=0;
+		size_t k; 
 		for (k=0; k<rX; k++) {
 			long double z = ((long double)X[k])*Y[k];
 			if (isnan(z)) {
@@ -365,19 +386,24 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 			cc += z;
 			nn++;
 		}	
-		CC[i+j*cX] = cc; 
+		CC[i] = cc; 
 		if (NN != NULL) 
-			NN[i+j*cX] = (double)nn; 
+			NN[i] = (double)nn; 
 	    }
 	else // if (X0==Y0) && (cX==cY)
 		/******** X==Y, output is symetric *******/	
 	    if (W) /* weighted version */
-	    for (i=0; i<cX; i++)
-	    for (j=i; j<cY; j++) {
-		X = X0+i*rX;
-		Y = Y0+j*rY;
+            #pragma omp for schedule(dynamic) nowait
+	    for (i = 0; i < cX * cY; i++)
+	    {
+		size_t ii = i%cX;
+		size_t jj = i/cX;
+	        if (ii < jj) continue;     
+		double *X = X0 + ii * rX;
+		double *Y = Y0 + jj * rY;
 		long double cc=0.0;
 		long double nn=0.0;
+		size_t k; 
 		for (k=0; k<rX; k++) {
 			long double z = ((long double)X[k])*Y[k];
 			if (isnan(z)) {
@@ -389,20 +415,26 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 			cc += z*W[k];
 			nn += W[k];
 		}	
-		CC[i+j*cX] = cc; 
-		CC[j+i*cX] = cc; 
+		size_t j = jj + ii*cX;
+		CC[i] = cc; 
+		CC[j] = cc; 
 		if (NN != NULL) {
-			NN[i+j*cX] = nn; 
-			NN[j+i*cX] = nn; 
+			NN[i] = nn; 
+			NN[j] = nn; 
 		}	
 	    }
 	    else /* no weights, all weights are 1 */
-	    for (i=0; i<cX; i++)
-	    for (j=i; j<cY; j++) {
-		X = X0+i*rX;
-		Y = Y0+j*rY;
+            #pragma omp for schedule(dynamic) nowait
+	    for (i = 0; i < cX * cY; i++)
+	    {
+		size_t ii = i%cX;
+		size_t jj = i/cX;
+	        if (ii < jj) continue;     
+		double *X = X0 + ii * rX;
+		double *Y = Y0 + jj * rY;
 		long double cc=0.0;
 		size_t nn=0;
+		size_t k; 
 		for (k=0; k<rX; k++) {
 			long double z = ((long double)X[k])*Y[k];
 			if (isnan(z)) {
@@ -414,11 +446,12 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 			cc += z;
 			nn++;
 		}	
-		CC[i+j*cX] = cc; 
-		CC[j+i*cX] = cc; 
+		size_t j = jj + ii*cX;
+		CC[i] = cc; 
+		CC[j] = cc; 
 		if (NN != NULL) {
-			NN[i+j*cX] = (double)nn; 
-			NN[j+i*cX] = (double)nn; 
+			NN[i] = (double)nn; 
+			NN[j] = (double)nn; 
 		}	
 	    }
 
@@ -436,14 +469,16 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 	if ( (X0 != Y0) || (cX != cY) )
 		/******** X!=Y, output is not symetric *******/	
 	    if (W) /* weighted version */
-	    for (i=0; i<cX; i++)
-	    for (j=0; j<cY; j++) {
-		X = X0+i*rX;
-		Y = Y0+j*rY;
+            #pragma omp for schedule(dynamic) nowait
+	    for (i = 0; i < cX * cY; i++)
+	    {
+		double *X = X0 + (i%cX) * rX;
+		double *Y = Y0 + (i/cX) * rY;
 		long double cc=0.0;
 		long double nn=0.0;
 		long double rc=0.0;
 		long double rn=0.0;
+		size_t k; 
 		for (k=0; k<rX; k++) {
 		        long double t,y; 
 			long double z = ((long double)X[k])*Y[k];
@@ -465,18 +500,20 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 			rn= (t-nn)-y;
 			nn= t; 
 		}	
-		CC[i+j*cX] = cc; 
+		CC[i] = cc; 
 		if (NN != NULL) 
-			NN[i+j*cX] = nn; 
+			NN[i] = nn; 
 	    }
 	    else /* no weights, all weights are 1 */
-  	    for (i=0; i<cX; i++)
-	    for (j=0; j<cY; j++) {
-		X = X0+i*rX;
-		Y = Y0+j*rY;
+            #pragma omp for schedule(dynamic) nowait
+	    for (i = 0; i < cX * cY; i++)
+	    {
+		double *X = X0 + (i%cX) * rX;
+		double *Y = Y0 + (i/cX) * rY;
 		long double cc=0.0;
 		long double rc=0.0;
 		size_t nn=0;
+		size_t k; 
 		for (k=0; k<rX; k++) {
 		        long double t,y; 
 			long double z = ((long double)X[k])*Y[k];
@@ -494,21 +531,26 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 
 			nn++;
 		}	
-		CC[i+j*cX] = cc; 
+		CC[i] = cc; 
 		if (NN != NULL) 
-			NN[i+j*cX] = (double)nn; 
+			NN[i] = (double)nn; 
 	    }
 	else // if (X0==Y0) && (cX==cY)
 		/******** X==Y, output is symetric *******/	
 	    if (W) /* weighted version */
-	    for (i=0; i<cX; i++)
-	    for (j=i; j<cY; j++) {
-		X = X0+i*rX;
-		Y = Y0+j*rY;
+            #pragma omp for schedule(dynamic) nowait
+	    for (i = 0; i < cX * cY; i++)
+	    {
+		size_t ii = i%cX;
+		size_t jj = i/cX;
+	        if (ii < jj) continue;     
+		double *X = X0 + ii * rX;
+		double *Y = Y0 + jj * rY;
 		long double cc=0.0;
 		long double nn=0.0;
 		long double rc=0.0;
 		long double rn=0.0;
+		size_t k; 
 		for (k=0; k<rX; k++) {
 		        long double t,y; 
 			long double z = ((long double)X[k])*Y[k];
@@ -530,21 +572,27 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 			rn= (t-nn)-y;
 			nn= t; 
 		}	
-		CC[i+j*cX] = cc; 
-		CC[j+i*cX] = cc; 
+		size_t j = jj + ii*cX;
+		CC[i] = cc; 
+		CC[j] = cc; 
 		if (NN != NULL) {
-			NN[i+j*cX] = nn; 
-			NN[j+i*cX] = nn; 
+			NN[i] = nn; 
+			NN[j] = nn; 
 		}	
 	    }
 	    else /* no weights, all weights are 1 */
-	    for (i=0; i<cX; i++)
-	    for (j=i; j<cY; j++) {
-		X = X0+i*rX;
-		Y = Y0+j*rY;
+            #pragma omp for schedule(dynamic) nowait
+	    for (i = 0; i < cX * cY; i++)
+	    {
+		size_t ii = i%cX;
+		size_t jj = i/cX;
+	        if (ii < jj) continue;     
+		double *X = X0 + ii * rX;
+		double *Y = Y0 + jj * rY;
 		long double cc=0.0;
 		long double rc=0.0;
 		size_t nn=0;
+		size_t k; 
 		for (k=0; k<rX; k++) {
 		        long double t,y; 
 			long double z = ((long double)X[k])*Y[k];
@@ -562,11 +610,12 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 
 			nn++;
 		}	
-		CC[i+j*cX] = cc; 
-		CC[j+i*cX] = cc; 
+		size_t j = jj + ii*cX;
+		CC[i] = cc; 
+		CC[j] = cc; 
 		if (NN != NULL) {
-			NN[i+j*cX] = (double)nn; 
-			NN[j+i*cX] = (double)nn; 
+			NN[i] = (double)nn; 
+			NN[j] = (double)nn; 
 		}	
 	    }
     }
@@ -583,14 +632,16 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 	if ( (X0 != Y0) || (cX != cY) )
 		/******** X!=Y, output is not symetric *******/	
 	    if (W) /* weighted version */
-	    for (i=0; i<cX; i++)
-	    for (j=0; j<cY; j++) {
-		X = X0+i*rX;
-		Y = Y0+j*rY;
+            #pragma omp for schedule(dynamic) nowait
+	    for (i = 0; i < cX * cY; i++)
+	    {
+		double *X = X0 + (i%cX) * rX;
+		double *Y = Y0 + (i/cX) * rY;
 		double cc=0.0;
 		double nn=0.0;
 		double rc=0.0;
 		double rn=0.0;
+		size_t k; 
 		for (k=0; k<rX; k++) {
 		        double t,y; 
 			double z = X[k]*Y[k];
@@ -612,18 +663,20 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 			rn= (t-nn)-y;
 			nn= t; 
 		}	
-		CC[i+j*cX] = cc; 
+		CC[i] = cc; 
 		if (NN != NULL) 
-			NN[i+j*cX] = nn; 
+			NN[i] = nn; 
 	    }
 	    else /* no weights, all weights are 1 */
-  	    for (i=0; i<cX; i++)
-	    for (j=0; j<cY; j++) {
-		X = X0+i*rX;
-		Y = Y0+j*rY;
+            #pragma omp for schedule(dynamic) nowait
+	    for (i = 0; i < cX * cY; i++)
+	    {
+		double *X = X0 + (i%cX) * rX;
+		double *Y = Y0 + (i/cX) * rY;
 		double cc=0.0;
 		double rc=0.0;
 		size_t nn=0;
+		size_t k; 
 		for (k=0; k<rX; k++) {
 		        double t,y; 
 			double z = X[k]*Y[k];
@@ -641,21 +694,26 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 
 			nn++;
 		}	
-		CC[i+j*cX] = cc; 
+		CC[i] = cc; 
 		if (NN != NULL) 
-			NN[i+j*cX] = (double)nn; 
+			NN[i] = (double)nn; 
 	    }
 	else // if (X0==Y0) && (cX==cY)
 		/******** X==Y, output is symetric *******/	
 	    if (W) /* weighted version */
-	    for (i=0; i<cX; i++)
-	    for (j=i; j<cY; j++) {
-		X = X0+i*rX;
-		Y = Y0+j*rY;
+            #pragma omp for schedule(dynamic) nowait
+	    for (i = 0; i < cX * cY; i++)
+	    {
+		size_t ii = i%cX;
+		size_t jj = i/cX;
+	        if (ii < jj) continue;     
+		double *X = X0 + ii * rX;
+		double *Y = Y0 + jj * rY;
 		double cc=0.0;
 		double nn=0.0;
 		double rc=0.0;
 		double rn=0.0;
+		size_t k; 
 		for (k=0; k<rX; k++) {
 		        double t,y; 
 			double z = X[k]*Y[k];
@@ -677,21 +735,27 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 			rn= (t-nn)-y;
 			nn= t; 
 		}	
-		CC[i+j*cX] = cc; 
-		CC[j+i*cX] = cc; 
+		size_t j = jj + ii*cX;
+		CC[i] = cc; 
+		CC[j] = cc; 
 		if (NN != NULL) {
-			NN[i+j*cX] = nn; 
-			NN[j+i*cX] = nn; 
+			NN[i] = nn; 
+			NN[j] = nn; 
 		}	
 	    }
 	    else /* no weights, all weights are 1 */
-	    for (i=0; i<cX; i++)
-	    for (j=i; j<cY; j++) {
-		X = X0+i*rX;
-		Y = Y0+j*rY;
+            #pragma omp for schedule(dynamic) nowait
+	    for (i = 0; i < cX * cY; i++)
+	    {
+		size_t ii = i%cX;
+		size_t jj = i/cX;
+	        if (ii < jj) continue;     
+		double *X = X0 + ii * rX;
+		double *Y = Y0 + jj * rY;
 		double cc=0.0;
 		double rc=0.0;
 		size_t nn=0;
+		size_t k; 
 		for (k=0; k<rX; k++) {
 		        double t,y; 
 			double z = X[k]*Y[k];
@@ -709,16 +773,19 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 
 			nn++;
 		}	
-		CC[i+j*cX] = cc; 
-		CC[j+i*cX] = cc; 
+		size_t j = jj + ii*cX;
+		CC[i] = cc; 
+		CC[j] = cc; 
 		if (NN != NULL) {
-			NN[i+j*cX] = (double)nn; 
-			NN[j+i*cX] = (double)nn; 
+			NN[i] = (double)nn; 
+			NN[j] = (double)nn; 
 		}	
 	    }
     }
 #endif
-
+   } // end pragma omg parallel 
+   
+   
 #ifndef NO_FLAG
 	//mexPrintf("Third argument must be not empty - otherwise status whether a NaN occured or not cannot be returned.");
 	/* this is a hack, the third input argument is used to return whether a NaN occured or not. 
