@@ -27,7 +27,7 @@
 //     HIS.X
 //     HIS.H
 //
-//    $Id: histo_mex.cpp 8223 2011-04-20 09:16:06Z schloegl $
+//    $Id: histo_mex.cpp 12790 2015-04-23 11:53:02Z schloegl $
 //    Copyright (C) 2009,2010,2011 Alois Schloegl <alois.schloegl@gmail.com>
 //    This function is part of the NaN-toolbox
 //    http://pub.ist.ac.at/~schloegl/matlab/NaN/
@@ -109,30 +109,24 @@ int compare(const void *a, const void *b) {
 			float f1,f2;
 			f1 = ((float*)Sort.Table)[ix1];
 			f2 = ((float*)Sort.Table)[ix2];
-			switch (isnan(f1) + 2*isnan(f2)) {
-				case 0:
-					if (f1<f2) z = -1; 
-					else if (f1>f2) z = 1; 
-				case 3:
-					break;
-				case 1: z = 1; break; 
-				case 2: z = -1; break; 
-				} 
+			z = isnan(f1) - isnan(f2);
+			if (z) break;
+			
+			if (f1<f2) z = -1; 
+			else if (f1>f2) z = 1; 
+			// else f1==f2 || (isnan(f1) && isnan(f2))	
 			break;
 			}
 		case mxDOUBLE_CLASS: {
 			double f1,f2;
 			f1 = ((double*)Sort.Table)[ix1];
 			f2 = ((double*)Sort.Table)[ix2];
-			switch (isnan(f1) + 2*isnan(f2)) {
-				case 0:
-					if (f1<f2) z = -1; 
-					else if (f1>f2) z = 1; 
-				case 3:
-					break;
-				case 1: z = 1; break; 
-				case 2: z = -1; break; 
-				} 
+			z = isnan(f1) - isnan(f2);
+			if (z) break;
+			
+			if (f1<f2) z = -1; 
+			else if (f1>f2) z = 1; 
+			// else f1==f2 || (isnan(f1) && isnan(f2))	
 			break;
 			}
 		case mxINT16_CLASS: {
