@@ -38,7 +38,7 @@
 // - sums of squares
 //
 //
-//    $Id: sumskipnan_mex.cpp 12492 2014-01-10 13:34:15Z schloegl $
+//    $Id: sumskipnan_mex.cpp 12826 2015-06-18 15:09:49Z schloegl $
 //    Copyright (C) 2009,2010,2011 Alois Schloegl <alois.schloegl@gmail.com>
 //    This function is part of the NaN-toolbox
 //    http://pub.ist.ac.at/~schloegl/matlab/NaN/
@@ -52,14 +52,14 @@
 #include <stdint.h>
 #include "mex.h"
 
-inline int __sumskipnan2w__(double *data, size_t Ni, double *s, double *No, char *flag_anyISNAN, double *W);
-inline int __sumskipnan3w__(double *data, size_t Ni, double *s, double *s2, double *No, char *flag_anyISNAN, double *W);
-inline int __sumskipnan2wr__(double *data, size_t Ni, double *s, double *No, char *flag_anyISNAN, double *W);
-inline int __sumskipnan3wr__(double *data, size_t Ni, double *s, double *s2, double *No, char *flag_anyISNAN, double *W);
-inline int __sumskipnan2we__(double *data, size_t Ni, double *s, double *No, char *flag_anyISNAN, double *W);
-inline int __sumskipnan3we__(double *data, size_t Ni, double *s, double *s2, double *No, char *flag_anyISNAN, double *W);
-inline int __sumskipnan2wer__(double *data, size_t Ni, double *s, double *No, char *flag_anyISNAN, double *W);
-inline int __sumskipnan3wer__(double *data, size_t Ni, double *s, double *s2, double *No, char *flag_anyISNAN, double *W);
+inline void __sumskipnan2w__(double *data, size_t Ni, double *s, double *No, char *flag_anyISNAN, double *W);
+inline void __sumskipnan3w__(double *data, size_t Ni, double *s, double *s2, double *No, char *flag_anyISNAN, double *W);
+inline void __sumskipnan2wr__(double *data, size_t Ni, double *s, double *No, char *flag_anyISNAN, double *W);
+inline void __sumskipnan3wr__(double *data, size_t Ni, double *s, double *s2, double *No, char *flag_anyISNAN, double *W);
+inline void __sumskipnan2we__(double *data, size_t Ni, double *s, double *No, char *flag_anyISNAN, double *W);
+inline void __sumskipnan3we__(double *data, size_t Ni, double *s, double *s2, double *No, char *flag_anyISNAN, double *W);
+inline void __sumskipnan2wer__(double *data, size_t Ni, double *s, double *No, char *flag_anyISNAN, double *W);
+inline void __sumskipnan3wer__(double *data, size_t Ni, double *s, double *s2, double *No, char *flag_anyISNAN, double *W);
 
 //#define NO_FLAG
 
@@ -98,10 +98,10 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 	        mexErrMsgTxt("SUMSKIPNAN.MEX has 1 to 3 output arguments.");
 
 	// get 1st argument
-	if(mxIsDouble(PInputs[0]) && !mxIsComplex(PInputs[0]))
+	if(mxIsDouble(PInputs[0]) && !mxIsComplex(PInputs[0]) && !mxIsSparse(PInputs[0]) )
 		LInput  = mxGetPr(PInputs[0]);
 	else 	
-		mexErrMsgTxt("First argument must be REAL/DOUBLE.");
+		mexErrMsgTxt("First argument must be and not sparse REAL/DOUBLE.");
 
     	// get 2nd argument
     	if  (PInputCount > 1) {
@@ -474,7 +474,7 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 }
 
 #define stride 1 
-inline int __sumskipnan2w__(double *data, size_t Ni, double *s, double *No, char *flag_anyISNAN, double *W)
+inline void __sumskipnan2w__(double *data, size_t Ni, double *s, double *No, char *flag_anyISNAN, double *W)
 {
 	long double sum=0; 
 	char   flag=0; 
@@ -529,7 +529,7 @@ inline int __sumskipnan2w__(double *data, size_t Ni, double *s, double *No, char
 }
 
 
-inline int __sumskipnan3w__(double *data, size_t Ni, double *s, double *s2, double *No, char *flag_anyISNAN, double *W)
+inline void __sumskipnan3w__(double *data, size_t Ni, double *s, double *s2, double *No, char *flag_anyISNAN, double *W)
 {
 	long double sum=0; 
 	long double msq=0; 
@@ -584,7 +584,7 @@ inline int __sumskipnan3w__(double *data, size_t Ni, double *s, double *s2, doub
 	*s2 = msq; 
 }
 
-inline int __sumskipnan2wr__(double *data, size_t Ni, double *s, double *No, char *flag_anyISNAN, double *W)
+inline void __sumskipnan2wr__(double *data, size_t Ni, double *s, double *No, char *flag_anyISNAN, double *W)
 {
 	double sum=0; 
 	char   flag=0; 
@@ -639,7 +639,7 @@ inline int __sumskipnan2wr__(double *data, size_t Ni, double *s, double *No, cha
 }
 
 
-inline int __sumskipnan3wr__(double *data, size_t Ni, double *s, double *s2, double *No, char *flag_anyISNAN, double *W)
+inline void __sumskipnan3wr__(double *data, size_t Ni, double *s, double *s2, double *No, char *flag_anyISNAN, double *W)
 {
 	double sum=0; 
 	double msq=0; 
@@ -706,7 +706,7 @@ inline int __sumskipnan3wr__(double *data, size_t Ni, double *s, double *s2, dou
         ACM Computing Surveys, Vol 23, No 1, March 1991. 
  ****************************************/
 
-inline int __sumskipnan2we__(double *data, size_t Ni, double *s, double *No, char *flag_anyISNAN, double *W)
+inline void __sumskipnan2we__(double *data, size_t Ni, double *s, double *No, char *flag_anyISNAN, double *W)
 {
 	long double sum=0; 
 	char   flag=0; 
@@ -778,7 +778,7 @@ inline int __sumskipnan2we__(double *data, size_t Ni, double *s, double *No, cha
 }
 
 
-inline int __sumskipnan3we__(double *data, size_t Ni, double *s, double *s2, double *No, char *flag_anyISNAN, double *W)
+inline void __sumskipnan3we__(double *data, size_t Ni, double *s, double *s2, double *No, char *flag_anyISNAN, double *W)
 {
 	long double sum=0; 
 	long double msq=0; 
@@ -860,7 +860,7 @@ inline int __sumskipnan3we__(double *data, size_t Ni, double *s, double *s2, dou
 	*s2 = msq; 
 }
 
-inline int __sumskipnan2wer__(double *data, size_t Ni, double *s, double *No, char *flag_anyISNAN, double *W)
+inline void __sumskipnan2wer__(double *data, size_t Ni, double *s, double *No, char *flag_anyISNAN, double *W)
 {
 	double sum=0; 
 	char   flag=0; 
@@ -932,7 +932,7 @@ inline int __sumskipnan2wer__(double *data, size_t Ni, double *s, double *No, ch
 }
 
 
-inline int __sumskipnan3wer__(double *data, size_t Ni, double *s, double *s2, double *No, char *flag_anyISNAN, double *W)
+inline void __sumskipnan3wer__(double *data, size_t Ni, double *s, double *s2, double *No, char *flag_anyISNAN, double *W)
 {
 	double sum=0; 
 	double msq=0; 
