@@ -1,6 +1,6 @@
 /*
 
-$Id: svm_model_matlab.c 8223 2011-04-20 09:16:06Z schloegl $
+$Id: svm_model_matlab.c 12776 2015-04-12 15:18:03Z schloegl $
 Copyright (c) 2000-2009 Chih-Chung Chang and Chih-Jen Lin
 Copyright (c) 2010 Alois Schloegl <alois.schloegl@gmail.com>
 This function is part of the NaN-toolbox
@@ -372,7 +372,7 @@ struct svm_model *matlab_matrix_to_model(const mxArray *matlab_struct, const cha
 		pprhs[0] = rhs[id];
 		if(mexCallMATLAB(1, pplhs, 1, pprhs, "transpose")) 
 		{
-			svm_destroy_model(model);
+			svm_free_and_destroy_model(&model);
 			*msg = "cannot transpose SV matrix";
 			return NULL;
 		}
