@@ -19,7 +19,7 @@
 % 
 % You should have received a copy of the GNU General Public License
 % along with this program; if not, write to the Free Software
-% Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+% Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
 
 
 clear 
@@ -30,7 +30,7 @@ c=[1:N]'*2>N;
 
 W3 = [ones(1,N/2)/5,ones(1,N/10)];
 for l=1:length(classifier),
-	fprintf(1,'%s\n',classifier(l));
+	fprintf(1,'%s\n',classifier{l});
 for k=1:10,
 
 x=randn(N,2);
